unit wsq_demo_main_form;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, ExtCtrls, LCLType; //LCLType for HBitmap

type

  { TMainForm }

  TMainForm = class(TForm)
    OpenButton: TButton;
    SaveButton: TButton;
    WSQtoStreamButton: TButton;
    RegisterButton: TButton;
    StreamToWSQButton: TButton;
    OutputImage: TImage;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure OpenButtonClick(Sender: TObject);
    procedure RegisterButtonClick(Sender: TObject);
    procedure SaveButtonClick(Sender: TObject);
    procedure StreamToWSQButtonClick(Sender: TObject);
    procedure WSQtoStreamButtonClick(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
    bitmap : HBitmap;
    open_dialog : TOpenDialog;
    save_dialog : TSaveDialog;
  end; 

{ WSQ library declarations }
function RegisterWSQ():Integer; cdecl; external 'WSQ_library64.dll' name 'RegisterWSQ';
function GenerateWSQLibrarySerialNumber():PAnsiChar; cdecl; external 'WSQ_library64.dll' name 'GenerateWSQLibrarySerialNumber';
function UnlockWSQLibrary(const authorization_code: PAnsiChar):Integer; cdecl; external 'WSQ_library64.dll' name 'UnlockWSQLibrary';

function CreateBMPFromFile (const file_name: PAnsiChar): HBitmap; cdecl; external 'WSQ_library64.dll' name 'CreateBMPFromFile';
function SaveBMPToFile(const bitmap: HBITMAP;  const filename: PAnsiChar; filetype: Integer):Integer; cdecl; external 'WSQ_library64.dll' name 'SaveBMPToFile';
function CreateBMPFromWSQByteArray (input_wsq_byte_array: PAnsiChar; const size_of_input_wsq_byte_array: Integer): HBitmap; cdecl; external 'WSQ_library64.dll' name 'CreateBMPFromWSQByteArray';
function SaveWSQByteArrayToImageFile(input_wsq_byte_array: PAnsiChar; const size_of_input_wsq_byte_array: Integer;  const filename: PAnsiChar; filetype: Integer):Integer; cdecl; external 'WSQ_library64.dll' name 'SaveWSQByteArrayToImageFile';
procedure SetShowFilePropertiesDialog(file_properties_dialog: Integer); cdecl; external 'WSQ_library64.dll' name 'SetShowFilePropertiesDialog';
procedure ShowFileConverter(); cdecl; external 'WSQ_library64.dll' name 'ShowFileConverter';
function WSQ_decode_stream(input_data_stream: PAnsiChar; const input_stream_length: Integer; output_data_stream: PPCHAR; width: PInteger; height: PInteger; ppi: PInteger; comment_text: PPCHAR):Integer; cdecl; external 'WSQ_library64.dll' name 'WSQ_decode_stream';
function WSQ_encode_stream(input_data_stream: PAnsiChar; const width: Integer; const height: Integer; const bitrate:Double;
				   const ppi: Integer; comment_text: PAnsiChar; output_data_stream: PPCHAR; output_stream_length: PInteger):Integer; cdecl; external 'WSQ_library64.dll' name 'WSQ_encode_stream';
function ConvertHBITMAPtoGrayScale256 (const bitmap: HBITMAP): HBitmap; cdecl; external 'WSQ_library64.dll' name 'ConvertHBITMAPtoGrayScale256';
function SaveHBITMAPtoFileAsGrayScale256BMP(const bitmap: HBITMAP;  const filename: PAnsiChar):Integer; cdecl; external 'WSQ_library64.dll' name 'SaveHBITMAPtoFileAsGrayScale256BMP';




var
  MainForm: TMainForm;

  iFileHandle: Integer;
  input_data_stream, output_data_stream: PAnsiChar;
  input_data_stream2, output_data_stream2: PAnsiChar;
  image_width, image_height, ppi: Integer;
  bitrate: Double;
  comment_text: PAnsiChar;
  ret, n, input_stream_length: Integer;
  output_stream2_length: Integer;

implementation

{ TMainForm }

procedure TMainForm.FormCreate(Sender: TObject);
begin
       open_dialog := TOpenDialog.Create(MainForm);
       save_dialog := TSaveDialog.Create(MainForm);
       bitmap := HBitmap(NIL);
       comment_text := PAnsiChar(NIL); // It is important to initialize to NIL to avoid crashing

end;


procedure TMainForm.FormDestroy(Sender: TObject);
begin
        open_dialog.Destroy;
        save_dialog.Destroy;
end;

procedure TMainForm.OpenButtonClick(Sender: TObject);
begin

    open_dialog.Filter := 'All images|*.wsq;*.bmp;*.tiff;*.tif;*.jpg;*.jpeg;*.jp2;*.jpc;' +
    '*.png;*.gif;*.tga;*.pcx;*.ppm;*.pgm;*.ico;*.wmf;*.wbmp;*.ska;*.mng;*.pnm;*.ras;*.raw;|' +
    'FBI Wavelet Scalar Quantization (*.wsq)|*.wsq|' +
    'Windows Bitmap Graphics (*.bmp)|*.bmp|' +
    'Macintosh Tagged Information File Format images (*.tiff)|*.tiff|' +
    'PC Tagged Information File Format images (*.tif)|*.tif|' +
    'Joint Photographic Experts Group images (*.jpg)|*.jpg|' +
    'Joint Photographic Experts Group images (*.jpeg)|*.jpeg|' +
    'JPEG2000 Part 1 images (*.jp2)|*.jp2|' +
    'JPEG2000 Code Stream images (*.jpc)|*.jpc|' +
    'Portable Network Graphics images (*.png)|*.png|' +
    'Compuserve Graphic Interchange Format images (*.gif)|*.gif|' +
    'Truevision Targa Graphic images (*.tga )|*.tga|' +
    'ZSoft Paintbrush images (PC Paintbrush Bitmap Graphic) (*.pcx)|*.pcx|' +
    'Portable Pixel Map Images (*.ppm)|*.ppm|' +
    'Portable Gray Map Images (*.pgm)|*.pgm|' +
    'Portable Bitmap Images (*.pbm)|*.pbm|' +
    'Windows Icons (*.ico)|*.ico|' +
    'Windows Metafiles (*.wmf)|*.wmf|' +
    'Wireless Bitmap File Format (*.wbmp)|*.wbmp|' +
    'SKA Format images (*.ska)|*.ska|' +
    'Muti-image Network Graphic Animation (*.mng)|*.mng|' +
    'PBM Portable Any Map Graphic Bitmap (*.pnm)|*.pnm|' +
    'Sun Graphic (*.ras)|*.ras|' +
    'RAW Graphic (*.raw)|*.raw';



if open_dialog.Execute() then
begin


// Take and use Handle of HBitmap directly from WSQ image library DLL.
// If the WSQ image library is called to load or save another picture, you will lose the current image
       OutputImage.Picture.Bitmap.Handle := CreateBMPFromFile( PAnsiChar(open_dialog.FileName));

        if ( OutputImage.Picture.Bitmap.Handle <> HBitmap(NIL)) then
        begin
           SaveButton.Enabled := True;
           OutputImage.Height := OutputImage.Picture.Bitmap.Height;
           OutputImage.Width := OutputImage.Picture.Bitmap.Width;
           Invalidate;
        end
        else
        begin
            Application.MessageBox(PCHAR('Unable to load file'), PCHAR('WSQ library error'), MB_OK or MB_ICONERROR );
        end;


{
//Make a copy of HBitmap into OutputImage->Picture

     bitmap := CreateBMPFromFile( PAnsiChar(open_dialog.FileName));

        if ( bitmap <> HBitmap(NIL)) then
        begin
           SaveButton.Enabled := True;
           OutputImage.Picture.Bitmap.Handle  := HBitmap(CopyImage(bitmap, IMAGE_BITMAP, 0, 0,LR_COPYRETURNORG));
           OutputImage.Height := OutputImage.Picture.Bitmap.Height;
           OutputImage.Width := OutputImage.Picture.Bitmap.Width;
           Invalidate;
        end
        else
        begin
            Application.MessageBox(PCHAR('Unable to load file'), PCHAR('WSQ library error'), MB_OK or MB_ICONERROR );
        end;
}



end;


end;

procedure TMainForm.RegisterButtonClick(Sender: TObject);
begin


// "RegisterWSQ64.exe" and "WSQ image library (for Windows 64-bit)" were compiled using Microsoft Visual C++ 2008 version compiler.
// If it fails to run then you need to install Microsoft Visual C++ 2008 Redistributable Package (x64).
// The download link is:
// https://www.microsoft.com/en-us/download/details.aspx?id=26368
// Microsoft Visual C++ 2008 Redistributable Package (x64)

RegisterWSQ();

end;

procedure TMainForm.SaveButtonClick(Sender: TObject);
begin

    save_dialog.Filter := 'WSQ Files (*.wsq)|*.wsq|' + // 1
'Windows BMP (*.bmp)|*.bmp|' + // 2
'TIFF Files (*.tif)|*.tif|' + // 3
'PNG Files (*.png)|*.png|' + // 4
'JPEG Files (*.jpg)|*.jpg|' + // 5
'Targa Files (*.tga)|*.tga|' + // 6
'SGI Files (*.rgb)|*.rgb|' + // 7
'JPEG-2000 Part-1 (*.jp2)|*.jp2|' + // 8
'JPEG-2000 Code Stream (*.jpc)|*.jpc|' + // 9
'PBM - Portable Bitmap Format (*.pbm)|*.pbm|' + // 10
'PGM - Portable Graymap Format (*.pgm)|*.pgm|' + // 11
'PPM - Portable Pixmap Format (*.ppm)|*.ppm|' + // 12
'GIF Files (*.gif)|*.gif|' + // 13
'PNM Files (*.pnm)|*.pnm|' + // 14
'RAS Files (*.ras)|*.ras|' + // 15
'MNG Files (*.mng)|*.mng|' + // 16
'PCX Files (*.pcx)|*.pcx'; // 17

    save_dialog.DefaultExt := 'wsq';


if save_dialog.Execute() then
begin

       SetShowFilePropertiesDialog(1);
       SaveBMPToFile( OutputImage.Picture.Bitmap.Handle, PAnsiChar(save_dialog.FileName), save_dialog.FilterIndex);

end;


end;

procedure TMainForm.StreamToWSQButtonClick(Sender: TObject);
var
    x, y, row, row2, pixel_number, pixel_number2: Integer;
begin

   save_dialog.Filter := 'WSQ Files (*.wsq)|*.wsq';
    save_dialog.DefaultExt := 'wsq';

if save_dialog.Execute() then
begin
    bitrate := 0.75;
    ppi := 500;
    comment_text := 'Cognaxon';

    if(input_data_stream2 <> PAnsiChar(nil)) then
    begin
    FreeMem(input_data_stream2); input_data_stream2 := PAnsiChar(nil);
    end;

    input_data_stream2 := PChar(AllocMem(image_width*image_height));


	// MAKE A MIRRORED/FLIPPED IMAGE (UP<->DOWN and LEFT<->RIGHT)
    for y := 0 to (image_height - 1) do
    begin
      row := image_width * y;
      row2 := image_width * (image_height - y - 1);

        for x := 0 to (image_width - 1) do
        begin
         pixel_number := row + x;
         pixel_number2 := row2 + (image_width - x - 1);

         input_data_stream2[pixel_number2] := output_data_stream[pixel_number];
        end;
    end;




// THE ENCODING OF DECOMPRESSED STREAM INTO WSQ STREAM
WSQ_encode_stream(input_data_stream2, image_width, image_height, bitrate, ppi, PAnsiChar(comment_text), @output_data_stream2, @output_stream2_length);

comment_text := PAnsiChar(NIL); // this prevents crashing when opening next WSQ image

    iFileHandle := FileCreate(save_dialog.FileName);
    FileWrite(iFileHandle, output_data_stream2^, output_stream2_length);
    FileClose(iFileHandle);

Application.MessageBox(PCHAR('Memory stream was compressed into another memory stream and saved to WSQ file.' + #13#10 + 'Now please press button "Open", select the saved file, open it and have a look at the' + #13#10 + 'results of memory stream processing. The processed image is mirrored/flipped' + #13#10 + '(UP<->DOWN and LEFT<->RIGHT).'), PCHAR('WSQ image library'), MB_OK );

end;


end;

procedure TMainForm.WSQtoStreamButtonClick(Sender: TObject);
var
  iFileLength: Integer;
begin


    open_dialog.Filter := 'FBI Wavelet Scalar Quantization (*.wsq)|*.wsq';


if open_dialog.Execute() then
begin

      iFileHandle := FileOpen(open_dialog.FileName, fmOpenRead);
      iFileLength := FileSeek(iFileHandle,0,2);
      FileSeek(iFileHandle,0,0);
      input_data_stream := PChar(AllocMem(iFileLength + 1));
      input_stream_length := FileRead(iFileHandle, input_data_stream^, iFileLength);
      FileClose(iFileHandle);


// THE DECODING OF WSQ STREAM INTO DECOMPRESSED STREAM
WSQ_decode_stream(input_data_stream, input_stream_length, @output_data_stream, @image_width, @image_height, @ppi, @comment_text);


//Converts WSQ compressed byte array into HBITMAP for showing on screen
       OutputImage.Picture.Bitmap.Handle := CreateBMPFromWSQByteArray(input_data_stream, input_stream_length);
// Saves WSQ compressed byte array to an image file.
       //SaveWSQByteArrayToImageFile(input_data_stream, input_stream_length, PCHAR('C:\OutputDirectory\output.jpg'), 5);


        if ( OutputImage.Picture.Bitmap.Handle <> HBitmap(NIL)) then
        begin
           SaveButton.Enabled := True;
           OutputImage.Height := OutputImage.Picture.Bitmap.Height;
           OutputImage.Width := OutputImage.Picture.Bitmap.Width;
           Invalidate;
           Application.MessageBox(PCHAR('WSQ file was loaded into memory stream and decompressed into another memory stream.' + #13#10 + 'Now please press button "stream -> WSQ" to compress memory stream and save it to file.' + #13#10 + 'WSQ compressed byte array was converted to HBITMAP for showing on screen.'), PCHAR('WSQ image library'), MB_OK );
        end
        else
        begin
            Application.MessageBox(PCHAR('Error: unable to decompress WSQ byte array.'), PCHAR('WSQ library error'), MB_OK or MB_ICONERROR );
        end;


       StreamToWSQButton.Enabled := True;
end;


end;

initialization
  {$I wsq_demo_main_form.lrs}

end.

